@@ -105,7 +105,20 @@ BASE_VERSION = 1
 # v1.65: Add lessee to the node object.
 # v1.66: Add support for node network_data field.
 # v1.67: Add support for port_uuid/portgroup_uuid in node vif_attach
-
+# v1.68: Add agent_verify_ca to heartbeat.
+# v1.69: Add deploy_steps to provisioning
+# v1.70: Add disable_ramdisk to manual cleaning.
+# v1.71: Add signifier for Scope based roles.
+# v1.72: Add agent_status and agent_status_message to /v1/heartbeat
+# v1.73: Add support for deploy and undeploy verbs
+# v1.74: Add bios registry to /v1/nodes/{node}/bios/{setting}
+# v1.75: Add boot_mode, secure_boot fields to node object.
+# v1.76: Add support for changing boot_mode and secure_boot state
+# v1.77: Add fields selector to drivers list and driver detail.
+# v1.78: Add node history endpoint
+# v1.79: Change allocation behaviour to prefer node name match
+# v1.80: Marker to represent self service node creation/deletion
+# v1.81: Add node inventory
 MINOR_0_JUNO = 0
 MINOR_1_INITIAL_VERSION = 1
 MINOR_2_AVAILABLE_STATE = 2
@@ -174,6 +187,20 @@ MINOR_64_LOCAL_LINK_CONNECTION_NETWORK_TYPE = 64
 MINOR_65_NODE_LESSEE = 65
 MINOR_66_NODE_NETWORK_DATA = 66
 MINOR_67_NODE_VIF_ATTACH_PORT = 67
+MINOR_68_HEARTBEAT_VERIFY_CA = 68
+MINOR_69_DEPLOY_STEPS = 69
+MINOR_70_CLEAN_DISABLE_RAMDISK = 70
+MINOR_71_RBAC_SCOPES = 71
+MINOR_72_HEARTBEAT_STATUS = 72
+MINOR_73_DEPLOY_UNDEPLOY_VERBS = 73
+MINOR_74_BIOS_REGISTRY = 74
+MINOR_75_NODE_BOOT_MODE = 75
+MINOR_76_NODE_CHANGE_BOOT_MODE = 76
+MINOR_77_DRIVER_FIELDS_SELECTOR = 77
+MINOR_78_NODE_HISTORY = 78
+MINOR_79_ALLOCATION_NODE_NAME = 79
+MINOR_80_PROJECT_CREATE_DELETE_NODE = 80
+MINOR_81_NODE_INVENTORY = 81
 
 # When adding another version, update:
 # - MINOR_MAX_VERSION
@@ -181,7 +208,7 @@ MINOR_67_NODE_VIF_ATTACH_PORT = 67
 #   explanation of what changed in the new version
 # - common/release_mappings.py, RELEASE_MAPPING['master']['api']
 
-MINOR_MAX_VERSION = MINOR_67_NODE_VIF_ATTACH_PORT
+MINOR_MAX_VERSION = MINOR_81_NODE_INVENTORY
 
 # String representations of the minor and maximum versions
 _MIN_VERSION_STRING = '{}.{}'.format(BASE_VERSION, MINOR_1_INITIAL_VERSION)
